/* **********************************************************
 * Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
 * **********************************************************/


/*
 * iocontrols.h
 *
 *        The driver io controls.
 */

#ifndef _IOCONTROLS_H_
#define _IOCONTROLS_H_

#define INCLUDE_ALLOW_USERLEVEL
#define INCLUDE_ALLOW_MONITOR
#define INCLUDE_ALLOW_MODULE
#define INCLUDE_ALLOW_VMKERNEL
#include "includeCheck.h"


/*
 * Driver version.
 *
 * Increment major version when you make an incompatible change.
 * Compatibility goes both ways (old driver with new executable
 * as well as new driver with old executable).
 */

#define VMMON_VERSION_GSX1	(3 << 16 | 0)
#define VMMON_VERSION_V3	(8 << 16 | 0)
#define VMMON_VERSION_V32	(10 << 16 | 0)
#define VMMON_VERSION_GSX2	(21 << 16 | 0)
#define VMMON_VERSION_GSX25	(41 << 16 | 0)
#define VMMON_VERSION_V4	(52 << 16 | 0)
#define VMMON_VERSION_MAJOR(v)	((uint32) (v) >> 16)
#define VMMON_VERSION_MINOR(v)	((uint16) (v))


/*
 * ENOMEM returned after MAX_VMS virtual machines created
 */

#ifdef VMX86_SERVER
#define MAX_VMS 64
#else
#define MAX_VMS 24
#endif

/*
 * See monitor_init.c for an explanation.
 */
#define VMX86_MAX_VCPUS 8


typedef enum IOCTLCmd {
   IOCTLCMD_GET_MHZ_ESTIMATE = 1,
   IOCTLCMD_NOT_IMPLEMENTED,	/* To get -EINVAL without error message */

   IOCTLCMD_NONE = 200,
   IOCTLCMD_VERSION,
   IOCTLCMD_CREATE_VM,
   IOCTLCMD_BIND_VM,
   IOCTLCMD_RELEASE_VM,
   IOCTLCMD_GET_NUM_VMS,
   IOCTLCMD_INIT_VM,
   IOCTLCMD_LATE_INIT_VM,
   IOCTLCMD_RUN_VM,
   IOCTLCMD_LOOK_UP_MPN,
   IOCTLCMD_LOCK_PAGE,
   IOCTLCMD_UNLOCK_PAGE,
   IOCTLCMD_APIC_BASE,
   IOCTLCMD_IOAPIC_BASE,
   IOCTLCMD_GET_STATS,
   IOCTLCMD_SET_STATS,
   IOCTLCMD_GET_HARD_LIMIT,
   IOCTLCMD_SET_HARD_LIMIT,
   IOCTLCMD_GET_MEM_INFO,
   IOCTLCMD_SET_MEM_INFO,
   IOCTLCMD_PAE_ENABLED,
   IOCTLCMD_GET_TOTAL_MEM_USAGE,
   IOCTLCMD_ACK_USER_CALL,
   IOCTLCMD_COMPLETE_USER_CALL,
   IOCTLCMD_GET_KHZ_ESTIMATE,
   /*
    * Keep host-specific calls at the end so they can be undefined
    * without renumbering the common calls.
    */

#ifdef linux
   IOCTLCMD_SET_UID,		// VMX86_DEVEL only
   IOCTLCMD_IS_MP_SAFE,
   IOCTLCMD_ALLOW_CORE_DUMP,

   IOCTLCMD_CHECK_MEMORY,

   IOCTLCMD_BROADCAST_IPI,	// SMP 2.2.8+ only

   IOCTLCMD_REGISTER_PASSTHROUGH_IO,
   IOCTLCMD_REGISTER_PASSTHROUGH_IRQ,
   IOCTLCMD_FREE_PASSTHROUGH_IO,
   IOCTLCMD_FREE_PASSTHROUGH_IRQ,
   IOCTLCMD_START_PASSTHROUGH,
   IOCTLCMD_STOP_PASSTHROUGH,
   IOCTLCMD_QUERY_PASSTHROUGH,

   IOCTLCMD_REGISTER_PERFCTR,
   IOCTLCMD_START_PERFCTR,
   IOCTLCMD_STOP_PERFCTR,
   IOCTLCMD_RELEASE_PERFCTR,

   IOCTLCMD_ALLOC_LOW_PAGES,
   IOCTLCMD_FREE_LOW_PAGES,

   IOCTLCMD_GET_SYSTEM_CLOCK_RATE,

#ifdef VMX86_SERVER
   IOCTLCMD_ALLOC_MACHINE_MEMORY,
   IOCTLCMD_POWERON_REQUEST_MEMORY,
   IOCTLCMD_ASYNC_ACTIONS,
   IOCTLCMD_FILE_OPEN,
   IOCTLCMD_FILE_CLOSE,
   IOCTLCMD_SET_MMAP_LAST,
   IOCTLCMD_INTERRUPT_WORLD,
   IOCTLCMD_DO_MEM_MAP,

   /* bora-root/fileutils/ls needs to be recompiled if the following
      definition ever changes */
   IOCTLCMD_VMFS_GET_LENGTH = 1000,
   IOCTLCMD_VMFS_PHYS_MEM_IO,

   /*
    * XXX I think this should be moved up to the first VMX86_SERVER
    * group.  -- edward
    */

   IOCTLCMD_ADD_WORLD_ARG,
#endif
#endif

#ifdef _WIN32
   IOCTLCMD_BEEP,
   IOCTLCMD_HARD_LIMIT_MONITOR_STATUS,	// Windows 2000 only
   IOCTLCMD_BLUE_SCREEN,	// USE_BLUE_SCREEN only
   IOCTLCMD_CHANGE_HARD_LIMIT,
   IOCTLCMD_ALLOC_LOW_PAGES,
   IOCTLCMD_FREE_LOW_PAGES,
   IOCTLCMD_SYNC_GET_TSCS,
   IOCTLCMD_SYNC_SET_TSCS,
#endif

   IOCTLCMD_LAST
} IOCTLCmd;


/*
 * Windows ioctl definitions.
 *
 * We use the IRP Information field for the return value
 * of IOCTLCMD_RUN, to be faster since it is use a lot.
 */

#ifdef _WIN32

#define FILE_DEVICE_VMX86        0x8101
#define VMX86_IOCTL_BASE_INDEX   0x801
#define VMIOCTL_BUFFERED(name) \
      CTL_CODE(FILE_DEVICE_VMX86, \
	       VMX86_IOCTL_BASE_INDEX+IOCTLCMD_##name, \
	       METHOD_BUFFERED, \
	       FILE_ANY_ACCESS)
#define VMIOCTL_NEITHER(name) \
      CTL_CODE(FILE_DEVICE_VMX86, \
	       VMX86_IOCTL_BASE_INDEX+IOCTLCMD_##name, \
	       METHOD_NEITHER, \
	       FILE_ANY_ACCESS)

#define IOCTL_VMX86_VERSION		VMIOCTL_BUFFERED(VERSION)
#define IOCTL_VMX86_CREATE_VM		VMIOCTL_BUFFERED(CREATE_VM)
#define IOCTL_VMX86_BIND_VM		VMIOCTL_BUFFERED(BIND_VM)
#define IOCTL_VMX86_RELEASE_VM		VMIOCTL_BUFFERED(RELEASE_VM)
#define IOCTL_VMX86_GET_NUM_VMS		VMIOCTL_BUFFERED(GET_NUM_VMS)
#define IOCTL_VMX86_INIT_VM		VMIOCTL_BUFFERED(INIT_VM)
#define IOCTL_VMX86_LATE_INIT_VM	VMIOCTL_BUFFERED(LATE_INIT_VM)
#define IOCTL_VMX86_RUN_VM		VMIOCTL_NEITHER(RUN_VM)
#define IOCTL_VMX86_LOOK_UP_MPN		VMIOCTL_BUFFERED(LOOK_UP_MPN)
#define IOCTL_VMX86_LOCK_PAGE		VMIOCTL_BUFFERED(LOCK_PAGE)
#define IOCTL_VMX86_UNLOCK_PAGE		VMIOCTL_BUFFERED(UNLOCK_PAGE)
#define IOCTL_VMX86_APIC_BASE		VMIOCTL_BUFFERED(APIC_BASE)
#define IOCTL_VMX86_IOAPIC_BASE		VMIOCTL_BUFFERED(IOAPIC_BASE)
#define IOCTL_VMX86_GET_STATS		VMIOCTL_BUFFERED(GET_STATS)
#define IOCTL_VMX86_SET_STATS		VMIOCTL_BUFFERED(SET_STATS)
#define IOCTL_VMX86_GET_HARD_LIMIT	VMIOCTL_BUFFERED(GET_HARD_LIMIT)
#define IOCTL_VMX86_SET_HARD_LIMIT	VMIOCTL_BUFFERED(SET_HARD_LIMIT)
#define IOCTL_VMX86_GET_MEM_INFO	VMIOCTL_BUFFERED(GET_MEM_INFO)
#define IOCTL_VMX86_SET_MEM_INFO	VMIOCTL_BUFFERED(SET_MEM_INFO)
#define IOCTL_VMX86_PAE_ENABLED		VMIOCTL_BUFFERED(PAE_ENABLED)

#define IOCTL_VMX86_BEEP		VMIOCTL_BUFFERED(BEEP)
#define IOCTL_VMX86_HARD_LIMIT_MONITOR_STATUS	VMIOCTL_BUFFERED(HARD_LIMIT_MONITOR_STATUS)
#define IOCTL_VMX86_CHANGE_HARD_LIMIT   VMIOCTL_BUFFERED(CHANGE_HARD_LIMIT)
#define IOCTL_VMX86_ALLOC_LOW_PAGES     VMIOCTL_BUFFERED(ALLOC_LOW_PAGES)
#define IOCTL_VMX86_FREE_LOW_PAGES      VMIOCTL_BUFFERED(FREE_LOW_PAGES)
#ifdef USE_BLUE_SCREEN
#define IOCTL_VMX86_BLUE_SCREEN		VMIOCTL_BUFFERED(BLUE_SCREEN)
#endif

#define IOCTL_VMX86_GET_VM_LIST		VMIOCTL_BUFFERED(GET_VM_LIST)
#define IOCTL_VMX86_GET_VM_INFO		VMIOCTL_BUFFERED(GET_VM_INFO)
#define IOCTL_VMX86_SET_VM_INFO		VMIOCTL_BUFFERED(SET_VM_INFO)
#define IOCTL_VMX86_GET_TOTAL_MEM_USAGE	VMIOCTL_BUFFERED(GET_TOTAL_MEM_USAGE)
#define IOCTL_VMX86_ACK_USER_CALL       VMIOCTL_BUFFERED(ACK_USER_CALL)
#define IOCTL_VMX86_COMPLETE_USER_CALL  VMIOCTL_BUFFERED(COMPLETE_USER_CALL)
#define IOCTL_VMX86_GET_KHZ_ESTIMATE    VMIOCTL_BUFFERED(GET_KHZ_ESTIMATE)
#define IOCTL_VMX86_SYNC_GET_TSCS       VMIOCTL_BUFFERED(SYNC_GET_TSCS)
#define IOCTL_VMX86_SYNC_SET_TSCS       VMIOCTL_BUFFERED(SYNC_SET_TSCS)

#endif


/*
 * Structure for GET_STATS and SET_STATS.
 */

typedef struct {
  uint32 userLockedPages;	/* current count of pages locked by app */
  uint32 maxUserLockedPages;	/* max over time of pages locked by app */
  uint32 monitorLockedPages;	/* current count of pages locked by monitor */
  uint32 maxMonitorLockedPages;	/* max over time of pages locked by monitor */
} VMX86Stats;


/*
 * Return codes from page locking, unlocking, and MPN lookup.
 * They share an error code space because they call one another
 * internally.
 *
 *    PAGE_LOCK_FAILED              The host refused to lock a page.
 *    PAGE_LOCK_LIMIT_EXCEEDED      We have reached the limit of locked
 *                                  pages for all VMs
 *    PAGE_LOCK_TOUCH_FAILED        Failed to touch page after lock.
 *    PAGE_LOCK_IN_TRANSITION       The page is locked but marked by Windows
 *                                  as nonpresent in CPU PTE and in transition 
 *                                  in Windows PFN.  
 *
 *    PAGE_LOCK_SYS_ERROR           System call error.
 *    PAGE_LOCK_ALREADY_LOCKED      Page already locked.
 *    PAGE_LOCK_MEMTRACKER_ERROR    MemTracker fails.
 *    PAGE_LOCK_PHYSTRACKER_ERROR   PhysTracker fails.
 *    PAGE_LOCK_MDL_ERROR           Mdl error on Windows.
 *
 *    PAGE_UNLOCK_NO_ERROR          Unlock successful (must be 0).
 *    PAGE_UNLOCK_NOT_TRACKED       Not in memtracker.
 *    PAGE_UNLOCK_NO_MPN            Tracked but no MPN.
 *    PAGE_UNLOCK_NOT_LOCKED        Not locked.
 *    PAGE_UNLOCK_TOUCH_FAILED      Failed to touch page.
 *
 *    PAGE_LOOKUP_INVALID_ADDR      Consistency checking on Windows.
 *    PAGE_LOOKUP_BAD_HIGH_ADDR     Consistency checking on Windows.
 *    PAGE_LOOKUP_ZERO_ADDR         Consistency checking on Windows.
 *    PAGE_LOOKUP_SMALL_ADDR        Consistency checking on Windows.
 *
 * PAGE_LOCK_ERROR_BIT is used to encode the error value as a positive
 * number to get through ioctl().
 *
 * -- edward
 */

#define PAGE_LOCK_FAILED                  -1
#define PAGE_LOCK_LIMIT_EXCEEDED          -2
#define PAGE_LOCK_TOUCH_FAILED            -3
#define PAGE_LOCK_IN_TRANSITION           -4

#define PAGE_LOCK_SYS_ERROR              -10
#define PAGE_LOCK_ALREADY_LOCKED         -11
#define PAGE_LOCK_MEMTRACKER_ERROR       -12
#define PAGE_LOCK_PHYSTRACKER_ERROR      -13
#define PAGE_LOCK_MDL_ERROR              -14

#define PAGE_UNLOCK_NO_ERROR		   0
#define PAGE_UNLOCK_NOT_TRACKED		-100
#define PAGE_UNLOCK_NO_MPN		-101
#define PAGE_UNLOCK_NOT_LOCKED		-102
#define PAGE_UNLOCK_TOUCH_FAILED	-103

#define PAGE_LOOKUP_INVALID_ADDR        -200
#define PAGE_LOOKUP_BAD_HIGH_ADDR       -201
#define PAGE_LOOKUP_ZERO_ADDR           -202
#define PAGE_LOOKUP_SMALL_ADDR          -203
#define PAGE_LOOKUP_NOT_TRACKED          -10	// added to another code
#define PAGE_LOOKUP_NO_MPN               -20	// added to another code
#define PAGE_LOOKUP_NOT_LOCKED           -30	// added to another code
#define PAGE_LOOKUP_NO_VM                -40	// added to another code

#define PAGE_LOCK_SUCCESS(mpn) ((int) (mpn) >= 0)
#define PAGE_LOCK_SOFT_FAILURE(mpn) ((int) (mpn) < 0 && (int) (mpn) > -10)

#define PAGE_LOCK_ERROR_BIT 0x4000000


/*
 * Flags sent into APICBASE ioctl
 */

#define APIC_FLAG_DISABLE_NMI       0x00000001
#define APIC_FLAG_PROBE             0x00000002
#define APIC_FLAG_FORCE_ENABLE      0x00000004


/*
 * Structure sent into REGISTER_PERFCOUNTERS ioctl
 */

#define PERFCTR_MAX_COUNTERS        2
#define PERFCTR_INVALID_EVENT       0
#define PERFCTR_INVALID_IRQ         -1

typedef struct PerfCtrInfo {
   uint32 eventNum;
   uint32 samplingRate;
} PerfCtrInfo;

typedef struct PerfCtrRegisterArgs {
   PerfCtrInfo counterInfo[PERFCTR_MAX_COUNTERS];
   int irq;
   Bool totalOnly;
} PerfCtrRegisterArgs;

typedef struct VMGetPCIInfoArgs {
   int busNumber;
   int deviceNumber;
   int functionNumber;
} VMGetPCIInfoArgs;

typedef struct VMGetPCIInfoResult {
   int bus;
   int slot;
   int vendorID;
   int deviceID;
   int baseClass;
   int subClass;
   int interruptLine;
   int interruptVector;
} VMGetPCIInfoResult;

typedef enum IOCTLCmdV2 { 
   IOCTLCMD_V2_NONE=200,
   IOCTLCMD_V2_INIT,
   IOCTLCMD_V2_LATE_INIT,
   IOCTLCMD_V2_RUN,
   IOCTLCMD_V2_LOOKUPMPN,
   IOCTLCMD_V2_BEEP,   // 205 
   IOCTLCMD_V2_SETUID, // Linux and VMX86_DEVEL only
   IOCTLCMD_V2_LOCKPAGE, 
   IOCTLCMD_V2_UNLOCKPAGE, 
   IOCTLCMD_V2_ISMPSAFE,
   IOCTLCMD_V2_APICBASE, // 210
   IOCTLCMD_V2_IOAPICBASE,
   IOCTLCMD_V2_REGISTER_PASSTHROUGH_IO,
   IOCTLCMD_V2_REGISTER_PASSTHROUGH_IRQ,
   IOCTLCMD_V2_FREE_PASSTHROUGH_IO,
   IOCTLCMD_V2_FREE_PASSTHROUGH_IRQ, // 215
   IOCTLCMD_V2_START_PASSTHROUGH,
   IOCTLCMD_V2_STOP_PASSTHROUGH,
   IOCTLCMD_V2_QUERY_PASSTHROUGH,
   IOCTLCMD_V2_GET_NUM_VMS,
   IOCTLCMD_V2_DECLARE_SLAVE, // 220
   IOCTLCMD_V2_CHECK_MEMORY,
   IOCTLCMD_V2_BLUE_SCREEN,
   IOCTLCMD_V2_ALLOW_CORE_DUMP,
   IOCTLCMD_V2_GETSTATS,
   IOCTLCMD_V2_SETSTATS, // 225
   IOCTLCMD_V2_SET_HARD_LIMIT,
   IOCTLCMD_V2_GET_HARD_LIMIT,
   IOCTLCMD_V2_GET_SET_MEM_INFO,
   IOCTLCMD_V2_GET_MEM_INFO,
   IOCTLCMD_V2_SET_MEM_INFO, // 230
   IOCTLCMD_V2_GET_VM_LIST,
   IOCTLCMD_V2_GET_VM_INFO,
   IOCTLCMD_V2_SET_VM_INFO,
   IOCTLCMD_V2_BROADCAST_IPI, // Linux SMP 2.2.8+ only
   IOCTLCMD_V2_FREE_RESOURCES,// Linux only, 235
   IOCTLCMD_V2_REDUCE_HARD_LIMIT, // Windows 2000 only
   IOCTLCMD_V2_LAST
} IOCTLCmdV2;

typedef enum IOCTLCmdV3 {
   IOCTLCMD_V3_NONE = 200,
   IOCTLCMD_V3_VERSION,
   IOCTLCMD_V3_CREATE_VM,
   IOCTLCMD_V3_BIND_VM,
   IOCTLCMD_V3_RELEASE_VM,
   IOCTLCMD_V3_GET_NUM_VMS,
   IOCTLCMD_V3_INIT_VM,
   IOCTLCMD_V3_LATE_INIT_VM,
   IOCTLCMD_V3_RUN_VM,
   IOCTLCMD_V3_LOOK_UP_MPN,
   IOCTLCMD_V3_LOCK_PAGE,
   IOCTLCMD_V3_UNLOCK_PAGE,
   IOCTLCMD_V3_APIC_BASE,
   IOCTLCMD_V3_IOAPIC_BASE,
   IOCTLCMD_V3_GET_STATS,
   IOCTLCMD_V3_SET_STATS,
   IOCTLCMD_V3_GET_HARD_LIMIT,
   IOCTLCMD_V3_SET_HARD_LIMIT,
   IOCTLCMD_V3_GET_MEM_INFO,
   IOCTLCMD_V3_SET_MEM_INFO,
   IOCTLCMD_V3_PAE_ENABLED,
   IOCTLCMD_V3_GET_TOTAL_MEM_USAGE,

   /*
    * Keep host-specific calls at the end so they can be undefined
    * without renumbering the common calls.
    */

   IOCTLCMD_V3_SET_UID,		// VMX86_DEVEL only
   IOCTLCMD_V3_IS_MP_SAFE,
   IOCTLCMD_V3_GET_MHZ_ESTIMATE,
   IOCTLCMD_V3_ALLOW_CORE_DUMP,

   IOCTLCMD_V3_CHECK_MEMORY,

   IOCTLCMD_V3_BROADCAST_IPI,	// SMP 2.2.8+ only

   IOCTLCMD_V3_REGISTER_PASSTHROUGH_IO,
   IOCTLCMD_V3_REGISTER_PASSTHROUGH_IRQ,
   IOCTLCMD_V3_FREE_PASSTHROUGH_IO,
   IOCTLCMD_V3_FREE_PASSTHROUGH_IRQ,
   IOCTLCMD_V3_START_PASSTHROUGH,
   IOCTLCMD_V3_STOP_PASSTHROUGH,
   IOCTLCMD_V3_QUERY_PASSTHROUGH,

   IOCTLCMD_V3_REGISTER_PERFCTR,
   IOCTLCMD_V3_START_PERFCTR,
   IOCTLCMD_V3_STOP_PERFCTR,
   IOCTLCMD_V3_RELEASE_PERFCTR,

   IOCTLCMD_V32_ALLOC_LOW_PAGES,
   IOCTLCMD_V32_FREE_LOW_PAGES,
} IOCTLCmdV3;

typedef enum IOCTLCmdGSX2 {
   IOCTLCMD_GSX2_NONE = 200,
   IOCTLCMD_GSX2_VERSION,
   IOCTLCMD_GSX2_CREATE_VM,
   IOCTLCMD_GSX2_BIND_VM,
   IOCTLCMD_GSX2_RELEASE_VM,
   IOCTLCMD_GSX2_GET_NUM_VMS,
   IOCTLCMD_GSX2_INIT_VM,
   IOCTLCMD_GSX2_LATE_INIT_VM,
   IOCTLCMD_GSX2_RUN_VM,
   IOCTLCMD_GSX2_LOOK_UP_MPN,
   IOCTLCMD_GSX2_LOCK_PAGE,
   IOCTLCMD_GSX2_UNLOCK_PAGE,
   IOCTLCMD_GSX2_APIC_BASE,
   IOCTLCMD_GSX2_IOAPIC_BASE,
   IOCTLCMD_GSX2_GET_STATS,
   IOCTLCMD_GSX2_SET_STATS,
   IOCTLCMD_GSX2_GET_HARD_LIMIT,
   IOCTLCMD_GSX2_SET_HARD_LIMIT,
   IOCTLCMD_GSX2_GET_MEM_INFO,
   IOCTLCMD_GSX2_SET_MEM_INFO,
   IOCTLCMD_GSX2_PAE_ENABLED,
   IOCTLCMD_GSX2_GET_TOTAL_MEM_USAGE,
   IOCTLCMD_GSX2_SET_UID,
   IOCTLCMD_GSX2_IS_MP_SAFE,
   IOCTLCMD_GSX2_GET_MHZ_ESTIMATE,
   IOCTLCMD_GSX2_ALLOW_CORE_DUMP,
   IOCTLCMD_GSX2_CHECK_MEMORY,
   IOCTLCMD_GSX2_BROADCAST_IPI,
   IOCTLCMD_GSX2_REGISTER_PASSTHROUGH_IO,
   IOCTLCMD_GSX2_REGISTER_PASSTHROUGH_IRQ,
   IOCTLCMD_GSX2_FREE_PASSTHROUGH_IO,
   IOCTLCMD_GSX2_FREE_PASSTHROUGH_IRQ,
   IOCTLCMD_GSX2_START_PASSTHROUGH,
   IOCTLCMD_GSX2_STOP_PASSTHROUGH,
   IOCTLCMD_GSX2_QUERY_PASSTHROUGH,
   IOCTLCMD_GSX2_REGISTER_PERFCTR,
   IOCTLCMD_GSX2_START_PERFCTR,
   IOCTLCMD_GSX2_STOP_PERFCTR,
   IOCTLCMD_GSX2_RELEASE_PERFCTR,
   IOCTLCMD_GSX2_ALLOC_LOW_PAGES,
   IOCTLCMD_GSX2_FREE_LOW_PAGES,
   IOCTLCMD_GSX2_LAST
} IOCTLCmdGSX2;

typedef enum IOCTLCmdGSX25 {
   IOCTLCMD_GSX25_NONE = 200,
   IOCTLCMD_GSX25_VERSION,
   IOCTLCMD_GSX25_CREATE_VM,
   IOCTLCMD_GSX25_BIND_VM,
   IOCTLCMD_GSX25_RELEASE_VM,
   IOCTLCMD_GSX25_GET_NUM_VMS,
   IOCTLCMD_GSX25_INIT_VM,
   IOCTLCMD_GSX25_LATE_INIT_VM,
   IOCTLCMD_GSX25_RUN_VM,
   IOCTLCMD_GSX25_LOOK_UP_MPN,
   IOCTLCMD_GSX25_LOCK_PAGE,
   IOCTLCMD_GSX25_UNLOCK_PAGE,
   IOCTLCMD_GSX25_APIC_BASE,
   IOCTLCMD_GSX25_IOAPIC_BASE,
   IOCTLCMD_GSX25_GET_STATS,
   IOCTLCMD_GSX25_SET_STATS,
   IOCTLCMD_GSX25_GET_HARD_LIMIT,
   IOCTLCMD_GSX25_SET_HARD_LIMIT,
   IOCTLCMD_GSX25_GET_MEM_INFO,
   IOCTLCMD_GSX25_SET_MEM_INFO,
   IOCTLCMD_GSX25_PAE_ENABLED,
   IOCTLCMD_GSX25_GET_TOTAL_MEM_USAGE,
   IOCTLCMD_GSX25_SYNC_GET_TSCS,
   IOCTLCMD_GSX25_SYNC_SET_TSCS,
   IOCTLCMD_GSX25_GET_KHZ_ESTIMATE,
   IOCTLCMD_GSX25_SET_UID,		// VMX86_DEVEL only
   IOCTLCMD_GSX25_IS_MP_SAFE,
   IOCTLCMD_GSX25_ALLOW_CORE_DUMP,
   IOCTLCMD_GSX25_CHECK_MEMORY,
   IOCTLCMD_GSX25_BROADCAST_IPI,	// SMP 2.2.8+ only
   IOCTLCMD_GSX25_REGISTER_PASSTHROUGH_IO,
   IOCTLCMD_GSX25_REGISTER_PASSTHROUGH_IRQ,
   IOCTLCMD_GSX25_FREE_PASSTHROUGH_IO,
   IOCTLCMD_GSX25_FREE_PASSTHROUGH_IRQ,
   IOCTLCMD_GSX25_START_PASSTHROUGH,
   IOCTLCMD_GSX25_STOP_PASSTHROUGH,
   IOCTLCMD_GSX25_QUERY_PASSTHROUGH,
   IOCTLCMD_GSX25_REGISTER_PERFCTR,
   IOCTLCMD_GSX25_START_PERFCTR,
   IOCTLCMD_GSX25_STOP_PERFCTR,
   IOCTLCMD_GSX25_RELEASE_PERFCTR,
   IOCTLCMD_GSX25_ALLOC_LOW_PAGES,
   IOCTLCMD_GSX25_FREE_LOW_PAGES,
   IOCTLCMD_GSX25_LAST
} IOCTLCmdGSX25;

#define VM_NAME_LENGTH        64
#define VM_USER_NAME_LENGTH   32

/*
 * Structure returned from GET_VM_LIST v2 ioctl.
 */
typedef struct {
   uint32   vmCount;
   uint32   vmUIDs[1];
} VMGetVMListResult;

typedef enum {
   VM_INFO_OFF,
   VM_INFO_RUNNING,
   VM_INFO_SUSPENDED,
   VM_INFO_BLOCKED,
} VMInfoState;

/*
 * Structure sent into VM_SET_INFO v2 ioctl.
 */
typedef struct {
   uint32		port;
   char			name[VM_NAME_LENGTH];
   char     		userName[VM_USER_NAME_LENGTH];
   VMInfoState 		state;
} VMSetVMInfoArgs;
/*
 * Structure returned from GET_VM_INFO v2 ioctl.
 */
typedef struct {
   uint32         vmUID;
   int32          port;
   char           name[VM_NAME_LENGTH];
   char           userName[VM_USER_NAME_LENGTH];
   VMInfoState	  state;
} VMGetVMInfoResult;

#ifdef VMX86_SERVER
#define FILE_LENGTH_ARGS_NAMELEN 256
/* Structure to get file length */
typedef struct VMFSFileLengthArgs {
   char fileName[FILE_LENGTH_ARGS_NAMELEN];
   long long length;
} VMFSFileLengthArgs;
#endif

typedef struct VMAPICInfo {
   uint32 flags;
   MA apicBase;
} VMAPICInfo; 

#ifndef MAX_PROCESSORS
#define MAX_PROCESSORS 32
#endif

typedef struct VMSyncWriteTSCs {
   uint32 cpuSet;
   uint32 TSCs[MAX_PROCESSORS];
} VMSyncWriteTSCs;


#define VMX86_DRIVER_VCPUID_OFFSET	1000

#endif // ifndef _IOCONTROLS_H_
